#define NUM_CELLS 40
#define NUM_CONFIG_BITS 20

typedef struct
{
	Xuint32 cellConfigs[NUM_CELLS];
	Xuint32 fitness;
} ArrayConfig;

//				Width		Write				Read
// Reg 0		(3)		arrayNum			arrayNum						+ 0
// Reg 1		(6)		cellNum			cellNum						+ 4
// Reg 2		(20)		config											+ 8
// Reg 3		(8)		vector			vector						+ 12
// Reg 4		(8)		mask				mask							+ 16
// Reg 5		(8)		ramData			ramData						+ 20
// Reg 6		(32)							output (3 downto 0)		+ 24
// Reg 7		(32)							output (7 downto 4)		+ 28
// Reg 8		(32)							match (3 downto 0)		+ 32
// Reg 9		(32)							match (7 downto 4)		+ 36
// Reg 10	(4)		DebugLEDS										+ 40

// EvoBlock Functions (write)
#define EVOBLOCK_writeArrayNum(data) \
	XIo_Out32(XPAR_OPB_EVOBLOCK_0_BASEADDR, (Xuint32)(data))

#define EVOBLOCK_writeCellNum(data) \
	XIo_Out32(XPAR_OPB_EVOBLOCK_0_BASEADDR+4, (Xuint32)(data))

#define EVOBLOCK_writeConfig(data) \
	XIo_Out32(XPAR_OPB_EVOBLOCK_0_BASEADDR+8, (Xuint32)(data))

#define EVOBLOCK_writeVector(data) \
	XIo_Out32(XPAR_OPB_EVOBLOCK_0_BASEADDR+12, (Xuint32)(data))

#define EVOBLOCK_writeMask(data) \
	XIo_Out32(XPAR_OPB_EVOBLOCK_0_BASEADDR+16, (Xuint32)(data))

#define EVOBLOCK_writeTargetRAM(data) \
	XIo_Out32(XPAR_OPB_EVOBLOCK_0_BASEADDR+20, (Xuint32)(data))

#define EVOBLOCK_setDebugLEDs(data) \
	XIo_Out32(XPAR_OPB_EVOBLOCK_0_BASEADDR+40, (Xuint32)(data))

// EvoBlock Functions (read)
#define EVOBLOCK_readArrayNum(data) \
	XIo_In32(XPAR_OPB_EVOBLOCK_0_BASEADDR)

#define EVOBLOCK_readCellNum(data) \
	XIo_In32(XPAR_OPB_EVOBLOCK_0_BASEADDR+4)

#define EVOBLOCK_readVector() \
	XIo_In32(XPAR_OPB_EVOBLOCK_0_BASEADDR+12)

#define EVOBLOCK_readMask() \
	XIo_In32(XPAR_OPB_EVOBLOCK_0_BASEADDR+16)

#define EVOBLOCK_readTargetRAM() \
	XIo_In32(XPAR_OPB_EVOBLOCK_0_BASEADDR+20)

#define EVOBLOCK_readOutput0() \
	XIo_In32(XPAR_OPB_EVOBLOCK_0_BASEADDR+24)

#define EVOBLOCK_readOutput1() \
	XIo_In32(XPAR_OPB_EVOBLOCK_0_BASEADDR+28)

#define EVOBLOCK_readMatch0() \
	XIo_In32(XPAR_OPB_EVOBLOCK_0_BASEADDR+32)

#define EVOBLOCK_readMatch1() \
	XIo_In32(XPAR_OPB_EVOBLOCK_0_BASEADDR+36)

Xuint32 createCellConfig(Xuint8 selC, Xuint8 selB, Xuint8 selA, Xuint8 lut);
void writeCellConfig(Xuint32 arrayNum, Xuint32 cellNum, Xuint32 config);
void configureArray(Xuint32 arrayNum, ArrayConfig *config);
void randomiseArrayConfig(ArrayConfig *config);
