library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity Cell is
	port (	clk			: in	std_logic;
				reset			: in	std_logic;
					
				dIn			: in	std_logic_vector(15 downto 0);
				dOut			: out	std_logic;

				configDIn	: in	std_logic_vector(19 downto 0);
				configWEn	: in	std_logic);
end Cell;

architecture Arch of Cell is

	signal dInCSelect		: std_logic_vector(3 downto 0);
	signal dInBSelect		: std_logic_vector(3 downto 0);
	signal dInASelect		: std_logic_vector(3 downto 0);
	signal lookUpTable	: std_logic_vector(7 downto 0);

	signal lutAddress		: std_logic_vector(2 downto 0);

	function mux8to1(	input	: std_logic_vector(7 downto 0);
							sel	: std_logic_vector(2 downto 0)) return std_logic is
		variable result : std_logic;
	begin
		case sel is
			when b"000"	=> result := input(0);
			when b"001"	=> result := input(1);
			when b"010"	=> result := input(2);
			when b"011"	=> result := input(3);
			when b"100"	=> result := input(4);
			when b"101"	=> result := input(5);
			when b"110"	=> result := input(6);
			when b"111"	=> result := input(7);
			when others	=> result := '0';
		end case;
		return result;
	end function mux8to1;

	function mux16to1(	input	: std_logic_vector(15 downto 0);
								sel	: std_logic_vector(3 downto 0)) return std_logic is
		variable result : std_logic;
	begin
		case sel is
			when b"0000"	=> result := input(0);
			when b"0001"	=> result := input(1);
			when b"0010"	=> result := input(2);
			when b"0011"	=> result := input(3);
			when b"0100"	=> result := input(4);
			when b"0101"	=> result := input(5);
			when b"0110"	=> result := input(6);
			when b"0111"	=> result := input(7);
			when b"1000"	=> result := input(8);
			when b"1001"	=> result := input(9);
			when b"1010"	=> result := input(10);
			when b"1011"	=> result := input(11);
			when b"1100"	=> result := input(12);
			when b"1101"	=> result := input(13);
			when b"1110"	=> result := input(14);
			when b"1111"	=> result := input(15);
			when others		=> result := '0';
		end case;
		return result;
	end function mux16to1;

begin

lutAddress(2) <= mux16to1(dIn, dInCSelect);
lutAddress(1) <= mux16to1(dIn, dInBSelect);
lutAddress(0) <= mux16to1(dIn, dInASelect);

dOut <= mux8to1(lookUpTable, lutAddress);

-- ConfigWrite ------------------------------------------------------------------------------------------------
-- Set the Cells parameters
---------------------------------------------------------------------------------------------------------------
ConfigWrite : process(clk)
begin

	if (clk'event and clk='1') then

		if (reset='1') then
			dInCSelect	<= (others => '0');
			dInBSelect	<= (others => '0');
			dInASelect	<= (others => '0');
			lookUpTable	<= (others => '0');

		elsif (configWEn='1') then
			dInCSelect	<= configDIn(19 downto 16);
			dInBSelect	<= configDIn(15 downto 12);
			dInASelect	<= configDIn(11 downto 8);
			lookUpTable	<= configDIn(7 downto 0);

		end if;

	end if;

end process ConfigWrite;

end Arch;
