library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity CellArray is
	generic ( NUM_COLS		: integer := 8);
	port (	clk				: in	std_logic;
				reset				: in	std_logic;

				dIn				: in	std_logic_vector(7 downto 0);
				dOut				: out	std_logic_vector(7 downto 0);

				cellNum			: in	integer range 0 to ((NUM_COLS * 8)-1);
				configDIn		: in	std_logic_vector(19 downto 0);
				configWEn		: in	std_logic);
end CellArray;

architecture Arch of CellArray is

	component Cell is
		port (	clk			: in	std_logic;
					reset			: in	std_logic;
					dIn			: in	std_logic_vector(15 downto 0);
					dOut			: out	std_logic;
					configDIn	: in	std_logic_vector(19 downto 0);
					configWEn	: in	std_logic);
	end component;

	subtype COL_RANGE is integer range 0 to (NUM_COLS-1);
	type COLUMN_OUTPUTS_TYPE is array (COL_RANGE) of std_logic_vector(7 downto 0);

	signal cellDOut : COLUMN_OUTPUTS_TYPE;

begin

dOut <= cellDOut(NUM_COLS-1);

-- ArrayGenerate ----------------------------------------------------------------------------------------------
-- Create the 8x8 array fo cells
---------------------------------------------------------------------------------------------------------------
ColGenerate : for colNum in COL_RANGE generate

	signal cellDIn : std_logic_vector(15 downto 0);

begin

	-- If first column
	Connect1 : if (colNum = 0) generate
		cellDIn(7 downto 0) <= not dIn;
		cellDIn(15 downto 8) <= dIn;
	end generate;

	-- If second columns
	Connect2 : if (colNum = 1) generate
		cellDIn(7 downto 0) <= dIn;
		cellDIn(15 downto 8) <= cellDOut(colNum-1);
	end generate;

	-- If other columns
	Connect3 : if (colNum > 1) generate
		cellDIn(7 downto 0) <= cellDOut(colNum-2);
		cellDIn(15 downto 8) <= cellDOut(colNum-1);
	end generate;

	RowGenerate : for rowNum in 0 to 7 generate

		constant cellNumber	: integer := rowNum + (colNum*8);
		signal cellConfigWEn	: std_logic;

	begin

		cellConfigWEn <= '1' when (cellNum=cellNumber and configWEn='1') else '0';

		CellInst : Cell
		port map (	clk			=> clk,
						reset			=> reset,
						dIn			=> cellDIn,
						dOut			=> cellDOut(colNum)(rowNum),
						configDIn	=> configDIn,
						configWEn	=> cellConfigWEn);

	end generate;

end generate;

end Arch;
